// ==========================================================================
// 
//  Logfile:    windows.h   
//
//  Desc:       This file provides portions of Windows compatability
//              not found in the standard MEWEL header files.
// 
//  Log:
//  Sat 06 Jan 1990   DavidHollifield
//      Create.
//
//  Tue 13 Feb 1990   DavidHollifield
//      - Added new WS_CAPTION which adds WIN_MOVEBOX capability.
//      - Added MoveWindow() macro.
//
// ==========================================================================

/* Notes -
 */

#if     !defined(H_WINDOWS)
#define H_WINDOWS

// ==========================================================================
//                               Header Files
// ==========================================================================
#include <window.h>
#include <mdi.h>

#undef   _CreateWindow

// ==========================================================================
//                               Definitions
// ==========================================================================

#define SW_SHOWNORMAL       SW_SHOW
#define BS_PUSHBUTTON       0x00L

// Window Style redefinitions
//
#undef  WS_CAPTION
#define WS_CAPTION	        (WIN_TITLEBAR | WIN_MOVEBOX)

#undef  LBS_STANDARD        
#define LBS_STANDARD        (LBS_NOTIFY | LBS_SORT | WS_VSCROLL | WS_BORDER)

// ==========================================================================
//                                Structures
// ==========================================================================

// TextMetric Structure
//
typedef struct tagTEXTMETRIC
{
    int 	tmHeight;
    int 	tmAscent;
    int 	tmDescent;
    int 	tmInternalLeading;
    int 	tmExternalLeading;
    int 	tmAveCharWidth;
    int 	tmMaxCharWidth;
    int 	tmWeight;
    BYTE	tmItalic;
    BYTE	tmUnderlined;
    BYTE	tmStruckOut;
    BYTE	tmFirstChar;
    BYTE	tmLastChar;
    BYTE	tmDefaultChar;
    BYTE	tmBreakChar;
    BYTE	tmPitchAndFamily;
    BYTE	tmCharSet;
    int 	tmOverhang;
    int 	tmDigitizedAspectX;
    int 	tmDigitizedAspectY;
} TEXTMETRIC;
typedef TEXTMETRIC	    *PTEXTMETRIC;
typedef TEXTMETRIC NEAR *NPTEXTMETRIC;
typedef TEXTMETRIC FAR	*LPTEXTMETRIC;

typedef DWORD   COLORREF;

// ==========================================================================
//                          External Declarations
// ==========================================================================

// ==========================================================================
//                            Data Declarations
// ==========================================================================

// ==========================================================================
//                           Function Prototypes
// ==========================================================================

// ==========================================================================
//                            Macro Declarations
// ==========================================================================

#define GetTextMetrics(hDC, ptm)                                            \
        {                                                                   \
            (ptm)->tmHeight           = 1;                                  \
            (ptm)->tmAscent           = 1;                                  \
            (ptm)->tmDescent          = 0;                                  \
            (ptm)->tmInternalLeading  = 0;                                  \
            (ptm)->tmExternalLeading  = 0;                                  \
            (ptm)->tmAveCharWidth     = 1;                                  \
            (ptm)->tmMaxCharWidth     = 1;                                  \
            (ptm)->tmWeight           = 0;                                  \
            (ptm)->tmItalic           = 0;                                  \
            (ptm)->tmUnderlined       = 0;                                  \
            (ptm)->tmStruckOut        = 0;                                  \
            (ptm)->tmFirstChar        = 0x00;                               \
            (ptm)->tmLastChar         = 0xFF;                               \
            (ptm)->tmDefaultChar      = 0x20;                               \
            (ptm)->tmBreakChar        = 0x0A;                               \
            (ptm)->tmPitchAndFamily   = 0;                                  \
            (ptm)->tmCharSet          = 0;                                  \
            (ptm)->tmOverhang         = 0;                                  \
            (ptm)->tmDigitizedAspectX = 0;                                  \
            (ptm)->tmDigitizedAspectY = 0;                                  \
        }

#endif

